# -*- coding: utf-8 -*-
from yookassa.domain.common import ResponseObject
from yookassa.domain.models import Amount, AuthorizationDetails, CancellationDetails, Recipient
from yookassa.domain.models.confirmation.confirmation_factory import ConfirmationFactory
from yookassa.domain.models.deal import PaymentDealInfo
from yookassa.domain.models.payment_data.payment_data_factory import PaymentDataFactory
from yookassa.domain.response.transfer_response import TransferResponse


class PaymentResponse(ResponseObject):
    """
    Class representing response object.

    Contains data
    """
    __id = None

    __status = None

    __amount = None

    __income_amount = None

    __description = None

    __recipient = None

    __payment_method = None

    __captured_at = None

    __created_at = None

    __expires_at = None

    __confirmation = None

    __test = None

    __refunded_amount = None

    __paid = None

    __refundable = None

    __receipt_registration = None

    __metadata = None

    __cancellation_details = None

    __authorization_details = None

    __transfers = None

    __deal = None

    __merchant_customer_id = None

    @property
    def id(self):
        return self.__id

    @id.setter
    def id(self, value):
        self.__id = value

    @property
    def status(self):
        return self.__status

    @status.setter
    def status(self, value):
        self.__status = value

    @property
    def amount(self):
        return self.__amount

    @amount.setter
    def amount(self, value):
        self.__amount = Amount(value)

    @property
    def income_amount(self):
        return self.__income_amount

    @income_amount.setter
    def income_amount(self, value):
        self.__income_amount = Amount(value)

    @property
    def description(self):
        return self.__description

    @description.setter
    def description(self, value):
        self.__description = value

    @property
    def recipient(self):
        return self.__recipient

    @recipient.setter
    def recipient(self, value):
        self.__recipient = Recipient(value)

    @property
    def payment_method(self):
        return self.__payment_method

    @payment_method.setter
    def payment_method(self, value):
        self.__payment_method = PaymentDataFactory().create(value, self.context())

    @property
    def captured_at(self):
        return self.__captured_at

    @captured_at.setter
    def captured_at(self, value):
        self.__captured_at = value

    @property
    def created_at(self):
        return self.__created_at

    @created_at.setter
    def created_at(self, value):
        self.__created_at = value

    @property
    def expires_at(self):
        return self.__expires_at

    @expires_at.setter
    def expires_at(self, value):
        self.__expires_at = value

    @property
    def confirmation(self):
        return self.__confirmation

    @confirmation.setter
    def confirmation(self, value):
        self.__confirmation = ConfirmationFactory().create(value, self.context())

    @property
    def test(self):
        return self.__test

    @test.setter
    def test(self, value):
        self.__test = bool(value)

    @property
    def refunded_amount(self):
        return self.__refunded_amount

    @refunded_amount.setter
    def refunded_amount(self, value):
        self.__refunded_amount = Amount(value)

    @property
    def paid(self):
        return self.__paid

    @paid.setter
    def paid(self, value):
        self.__paid = bool(value)

    @property
    def refundable(self):
        return self.__refundable

    @refundable.setter
    def refundable(self, value):
        self.__refundable = bool(value)

    @property
    def receipt_registration(self):
        return self.__receipt_registration

    @receipt_registration.setter
    def receipt_registration(self, value):
        self.__receipt_registration = value

    @property
    def metadata(self):
        return self.__metadata

    @metadata.setter
    def metadata(self, value):
        self.__metadata = value

    @property
    def cancellation_details(self):
        return self.__cancellation_details

    @cancellation_details.setter
    def cancellation_details(self, value):
        self.__cancellation_details = CancellationDetails(value)

    @property
    def authorization_details(self):
        return self.__authorization_details

    @authorization_details.setter
    def authorization_details(self, value):
        self.__authorization_details = AuthorizationDetails(value)

    @property
    def transfers(self):
        return self.__transfers

    @transfers.setter
    def transfers(self, value):
        if isinstance(value, list):
            self.__transfers = [TransferResponse(item) for item in value]
        elif value is None:
            self.__transfers = value
        else:
            raise TypeError('Invalid transfers data type in payment_response.transfers')

    @property
    def deal(self):
        return self.__deal

    @deal.setter
    def deal(self, value):
        self.__deal = PaymentDealInfo(value)

    @property
    def merchant_customer_id(self):
        return self.__merchant_customer_id

    @merchant_customer_id.setter
    def merchant_customer_id(self, value):
        self.__merchant_customer_id = value



# -*- coding: utf-8 -*-
from yookassa.domain.common.confirmation_type import ConfirmationType
from yookassa.domain.models.confirmation.confirmation import Confirmation


class ConfirmationQr(Confirmation):
    """
    Class representing qr confirmation data object
    """
    __confirmation_data = None

    def __init__(self, *args, **kwargs):
        super(ConfirmationQr, self).__init__(*args, **kwargs)
        if self.type is None or self.type is not ConfirmationType.QR:
            self.type = ConfirmationType.QR

    @property
    def confirmation_data(self):
        return self.__confirmation_data

    @confirmation_data.setter
    def confirmation_data(self, value):
        cast_value = str(value)
        if cast_value:
            self.__confirmation_data = cast_value
        else:
            raise ValueError('Invalid confirmation_data value')



# -*- coding: utf-8 -*-
from deprecated.classic import deprecated
from yookassa.domain.common import BaseObject


@deprecated("This class will be removed in one of future versions")
class RequestorType(object):
    MERCHANT = "merchant"
    THIRD_PARTY_CLIENT = "third_party_client"


@deprecated("This class will be removed in one of future versions")
class Requestor(BaseObject):
    """
    Class representing requestor data wrapper object
    """
    __type = None

    @property
    def type(self):
        return self.__type

    @type.setter
    def type(self, value):
        self.__type = str(value)


@deprecated("This class will be removed in one of future versions")
class RequestorMerchant(Requestor):
    """
    Class representing requestor_merchant data wrapper object
    """
    __type = RequestorType.MERCHANT

    __account_id = None

    @property
    def account_id(self):
        return self.__account_id

    @account_id.setter
    def account_id(self, value):
        self.__account_id = str(value)


@deprecated("This class will be removed in one of future versions")
class RequestorThirdPartyClient(Requestor):
    """
    Class representing requestor_third_party_client data wrapper object
    """
    __type = RequestorType.THIRD_PARTY_CLIENT

    __client_id = None

    __client_name = None

    @property
    def client_id(self):
        return self.__client_id

    @client_id.setter
    def client_id(self, value):
        self.__client_id = str(value)

    @property
    def client_name(self):
        return self.__client_name

    @client_name.setter
    def client_name(self, value):
        self.__client_name = str(value)


@deprecated("This class will be removed in one of future versions")
class RequestorFactory(object):
    """
    Base factory class for object that has type property.
    """

    def create(self, data):
        """
        Create instance from value

        :param data: dictionary that has type key
        :return: Typed object instance
        """
        if isinstance(data, dict):
            if 'type' in data:
                return self.__get_instance(data)
            else:
                raise ValueError('Parameter "data" should contain "type" field')
        else:
            raise TypeError('Parameter "data" should be "dict"')

    def __get_instance(self, data):
        class_object = self.__get_class_object(data)
        return class_object(data)

    @staticmethod
    def __get_class_object(data):
        if data['type'] == RequestorType.MERCHANT:
            return RequestorMerchant
        elif data['type'] == RequestorType.THIRD_PARTY_CLIENT:
            return RequestorThirdPartyClient
        else:
            raise TypeError('Parameter "data" should contain "type" field')



# -*- coding: utf-8 -*-
from yookassa.domain.common import BaseObject
from yookassa.domain.common.payment_method_type import PaymentMethodType
from yookassa.domain.models.payment_data.payment_data import ResponsePaymentData
from yookassa.domain.models.payment_data.request.payment_data_b2b_sberbank import VatData


class PaymentDataB2bSberbank(ResponsePaymentData):
    __payment_purpose = None

    __vat_data = None

    __payer_bank_details = None

    def __init__(self, *args, **kwargs):
        super(PaymentDataB2bSberbank, self).__init__(*args, **kwargs)
        if self.type is None or self.type is not PaymentMethodType.B2B_SBERBANK:
            self.type = PaymentMethodType.B2B_SBERBANK

    @property
    def payment_purpose(self):
        return self.__payment_purpose

    @payment_purpose.setter
    def payment_purpose(self, value):
        self.__payment_purpose = str(value)

    @property
    def vat_data(self):
        return self.__vat_data

    @vat_data.setter
    def vat_data(self, value):
        if isinstance(value, dict):
            self.__vat_data = VatData(value)
        elif isinstance(value, VatData):
            self.__vat_data = value
        else:
            raise TypeError('Invalid vat_data value type in PaymentDataB2bSberbank')

    @property
    def payer_bank_details(self):
        return self.__payer_bank_details

    @payer_bank_details.setter
    def payer_bank_details(self, value):
        if isinstance(value, dict):
            self.__payer_bank_details = PayerBankDetails(value)
        elif isinstance(value, PayerBankDetails):
            self.__payer_bank_details = value
        else:
            raise TypeError('Invalid payer_bank_details value type in PaymentDataB2bSberbank')


class PayerBankDetails(BaseObject):
    __full_name = None

    __short_name = None

    __address = None

    __inn = None

    __kpp = None

    __bank_name = None

    __bank_branch = None

    __bank_bik = None

    __account = None

    @property
    def full_name(self):
        return self.__full_name

    @full_name.setter
    def full_name(self, value):
        self.__full_name = str(value)

    @property
    def short_name(self):
        return self.__short_name

    @short_name.setter
    def short_name(self, value):
        self.__short_name = str(value)

    @property
    def address(self):
        return self.__address

    @address.setter
    def address(self, value):
        self.__address = str(value)

    @property
    def inn(self):
        return self.__inn

    @inn.setter
    def inn(self, value):
        self.__inn = str(value)

    @property
    def kpp(self):
        return self.__inn

    @kpp.setter
    def kpp(self, value):
        self.__inn = str(value)

    @property
    def bank_name(self):
        return self.__bank_name

    @bank_name.setter
    def bank_name(self, value):
        self.__bank_name = str(value)

    @property
    def bank_branch(self):
        return self.__bank_branch

    @bank_branch.setter
    def bank_branch(self, value):
        self.__bank_branch = str(value)

    @property
    def bank_bik(self):
        return self.__bank_bik

    @bank_bik.setter
    def bank_bik(self, value):
        self.__bank_bik = str(value)

    @property
    def account(self):
        return self.__account

    @account.setter
    def account(self, value):
        self.__account = str(value)



# -*- coding: utf-8 -*-
from yookassa.domain.common import BaseObject
from yookassa.domain.models.receipt_customer import ReceiptCustomer
from yookassa.domain.models.receipt_item import ReceiptItem


class Receipt(BaseObject):
    """
    Class representing receipt data wrapper object
    """
    __customer = None

    __items = []

    __tax_system_code = None

    @property
    def customer(self):
        return self.__customer

    @customer.setter
    def customer(self, value):
        if isinstance(value, dict):
            self.__customer = ReceiptCustomer(value)
        elif isinstance(value, ReceiptCustomer):
            self.__customer = value
        else:
            raise TypeError('Invalid customer value type')

    @property
    def items(self):
        return self.__items

    @items.setter
    def items(self, value):
        if isinstance(value, list):
            items = []
            for item in value:
                if isinstance(item, dict):
                    items.append(ReceiptItem(item))
                elif isinstance(item, ReceiptItem):
                    items.append(item)
                else:
                    raise TypeError('Invalid item type in receipt.items')

            self.__items = items
        elif value is None:
            self.__items = []
        else:
            raise TypeError('Invalid items value type in receipt')

    @property
    def tax_system_code(self):
        return self.__tax_system_code

    @tax_system_code.setter
    def tax_system_code(self, value):
        if isinstance(value, int):
            self.__tax_system_code = value
        else:
            raise TypeError('Invalid tax_system_code value type')

    @property
    def email(self):
        return self.__customer.email if self.__customer is not None else None

    @email.setter
    def email(self, value):
        if self.__customer is None:
            self.__customer = ReceiptCustomer()
        self.__customer.email = str(value)

    @property
    def phone(self):
        return self.__customer.phone if self.__customer is not None else None

    @phone.setter
    def phone(self, value):
        if self.__customer is None:
            self.__customer = ReceiptCustomer()
        self.__customer.phone = str(value)

    def has_items(self):
        return bool(self.items)



# -*- coding: utf-8 -*-
from yookassa.domain.common.request_object import RequestObject


class WebhookRequest(RequestObject):
    __event = None

    __url = None

    @property
    def event(self):
        return self.__event

    @event.setter
    def event(self, value):
        cast_value = str(value)
        self.__event = cast_value

    @property
    def url(self):
        return self.__url

    @url.setter
    def url(self, value):
        cast_value = str(value)
        self.__url = cast_value



# -*- coding: utf-8 -*-
from yookassa.domain.common.payment_method_type import PaymentMethodType
from yookassa.domain.models.payout_data.payout_destination import PayoutDestination


class PayoutDestinationYooMoneyWallet(PayoutDestination):

    __account_number = None

    def __init__(self, *args, **kwargs):
        super(PayoutDestinationYooMoneyWallet, self).__init__(*args, **kwargs)
        if self.type is None or self.type is not PaymentMethodType.YOO_MONEY:
            self.type = PaymentMethodType.YOO_MONEY

    @property
    def account_number(self):
        return self.__account_number

    @account_number.setter
    def account_number(self, value):
        self.__account_number = str(value)



# -*- coding: utf-8 -*-
from yookassa.domain.common.request_object import RequestObject
from yookassa.domain.models.airline import Airline
from yookassa.domain.models.amount import Amount
from yookassa.domain.models.receipt import Receipt
from yookassa.domain.models.transfer import Transfer


class CapturePaymentRequest(RequestObject):

    __amount = None

    __receipt = None

    __airline = None

    __transfers = []

    @property
    def amount(self):
        return self.__amount

    @amount.setter
    def amount(self, value):
        if isinstance(value, dict):
            self.__amount = Amount(value)
        elif isinstance(value, Amount):
            self.__amount = value
        else:
            raise TypeError('Invalid amount value type')

    @property
    def receipt(self):
        return self.__receipt

    @receipt.setter
    def receipt(self, value):
        if isinstance(value, dict):
            self.__receipt = Receipt(value)
        elif isinstance(value, Receipt):
            self.__receipt = value
        else:
            raise TypeError('Invalid receipt value type')

    @property
    def airline(self):
        return self.__airline

    @airline.setter
    def airline(self, value):
        if isinstance(value, dict):
            self.__airline = Airline(value)
        elif isinstance(value, Airline):
            self.__airline = value
        else:
            raise TypeError('Invalid airline type')

    @property
    def transfers(self):
        return self.__transfers

    @transfers.setter
    def transfers(self, value):
        if isinstance(value, list):
            self.__transfers = [Transfer(item) for item in value]
        elif value is None:
            self.__transfers = []
        else:
            raise TypeError('Invalid transfers data type in capture_payment_request.transfers')

    def validate(self):
        if self.amount:
            value = self.amount.value
            if not value or value <= 0.0:
                self.__set_validation_error('Invalid amount value: ' + str(value))
        if self.receipt is not None and self.receipt.has_items:
            email = self.receipt.email
            phone = self.receipt.phone
            if not email and not phone:
                self.__set_validation_error('Both email and phone values are empty in receipt')

            if not self.receipt.tax_system_code and any(not item.vat_code for item in self.receipt.items):
                self.__set_validation_error('Item vat_id and receipt tax_system_id not specified')

    def __set_validation_error(self, message):
        raise ValueError(message)



# -*- coding: utf-8 -*-
from yookassa.domain.common.response_object import ResponseObject
from yookassa.domain.response.receipt_response import ReceiptResponse


class ReceiptListResponse(ResponseObject):

    __type = None

    __items = None

    __next_cursor = None

    @property
    def type(self):
        return self.__type

    @type.setter
    def type(self, value):
        self.__type = value

    @property
    def next_cursor(self):
        return self.__next_cursor

    @next_cursor.setter
    def next_cursor(self, value):
        self.__next_cursor = value

    @property
    def items(self):
        return self.__items

    @items.setter
    def items(self, value):
        if isinstance(value, list):
            self.__items = [ReceiptResponse(receipt) for receipt in value]
        else:
            self.__items = value



# -*- coding: utf-8 -*-
from yookassa.domain.common import BaseObject


class Confirmation(BaseObject):
    """
    Base class confirmation data objects
    """
    __type = None

    @property
    def type(self):
        return self.__type

    @type.setter
    def type(self, value):
        self.__type = str(value)



# -*- coding: utf-8 -*-
from yookassa.client import ApiClient
from yookassa.domain.common.http_verb import HttpVerb


class Settings:
    base_path = '/me'

    def __init__(self):
        self.client = ApiClient()

    @classmethod
    def get_account_settings(cls, params=None):
        """
        Shop Info

        :param params: (dict | None) Параметры поиска.
            В настоящее время доступен только {'on_behalf_of': account_id}
        :return: dict
        """
        instance = cls()
        path = cls.base_path

        response = instance.client.request(HttpVerb.GET, path, params)
        return response



# -*- coding: utf-8 -*-
from netaddr import IPNetwork, IPAddress


class SecurityHelper:
    """
    """

    def __init__(self):
        pass

    YOOKASSA_NETWORKS = [
        '77.75.153.0/25',
        '77.75.156.11',
        '77.75.156.35',
        '77.75.154.128/25',
        '185.71.76.0/27',
        '185.71.77.0/27',
        '2a02:5180:0:1509::/64',
        '2a02:5180:0:2655::/64',
        '2a02:5180:0:1533::/64',
        '2a02:5180:0:2669::/64'
    ]

    @staticmethod
    def is_ip_in_network(ip, network):
        return IPAddress(ip) in IPNetwork(network)

    def is_ip_trusted(self, ip):
        return any(map(lambda net: self.is_ip_in_network(ip, net), self.YOOKASSA_NETWORKS))



# -*- coding: utf-8 -*-
from yookassa.domain.request.payment_request import PaymentRequest


class PaymentRequestBuilder(object):
    def __init__(self):
        self.__request = PaymentRequest()

    def set_recipient(self, value):
        self.__request.recipient = value
        return self

    def set_amount(self, value):
        self.__request.amount = value
        return self

    def set_description(self, value):
        self.__request.description = value
        return self

    def set_receipt(self, value):
        self.__request.receipt = value
        return self

    def set_payment_token(self, value):
        self.__request.payment_token = value
        return self

    def set_payment_method_id(self, value):
        self.__request.payment_method_id = value
        return self

    def set_payment_method_data(self, value):
        self.__request.payment_method_data = value
        return self

    def set_confirmation(self, value):
        self.__request.confirmation = value
        return self

    def set_save_payment_method(self, value):
        self.__request.save_payment_method = value
        return self

    def set_capture(self, value):
        self.__request.capture = value
        return self

    def set_client_ip(self, value):
        self.__request.client_ip = value
        return self

    def set_airline(self, value):
        self.__request.airline = value
        return self

    def set_metadata(self, value):
        self.__request.metadata = value
        return self

    def set_transfers(self, value):
        self.__request.transfers = value
        return self

    def set_deal(self, value):
        self.__request.deal = value
        return self

    def set_merchant_customer_id(self, value):
        self.__request.merchant_customer_id = value
        return self

    def build(self):
        return self.__request



# -*- coding: utf-8 -*-
from yookassa.domain.common.request_object import RequestObject
from yookassa.domain.models.amount import Amount
from yookassa.domain.models.deal import PayoutDealInfo
from yookassa.domain.models.payout_data.payout_destination import PayoutDestination
from yookassa.domain.models.payout_data.payout_destination_factory import PayoutDestinationFactory

DESCRIPTION_MAX_LENGTH = 128


class PayoutRequest(RequestObject):
    """
    Class representing PayoutRequest values enum
    """

    __amount = None

    __description = None

    __payout_destination_data = None

    __payout_token = None

    __deal = None

    __metadata = None

    @property
    def amount(self):
        return self.__amount

    @amount.setter
    def amount(self, value):
        if isinstance(value, dict):
            self.__amount = Amount(value)
        elif isinstance(value, Amount):
            self.__amount = value
        else:
            raise TypeError('Invalid amount value type')

    @property
    def description(self):
        return self.__description

    @property
    def payout_destination_data(self):
        return self.__payout_destination_data

    @payout_destination_data.setter
    def payout_destination_data(self, value):
        if isinstance(value, dict):
            self.__payout_destination_data = PayoutDestinationFactory().create(value, self.context())
        elif isinstance(value, PayoutDestination):
            self.__payout_destination_data = value
        else:
            raise TypeError('Invalid payout_destination_data type')

    @property
    def payout_token(self):
        return self.__payout_token

    @payout_token.setter
    def payout_token(self, value):
        cast_value = str(value)
        if cast_value:
            self.__payout_token = cast_value
        else:
            raise TypeError('Invalid payout_token value')

    @description.setter
    def description(self, value):
        cast_value = str(value)
        if cast_value:
            if len(cast_value) <= DESCRIPTION_MAX_LENGTH:
                self.__description = cast_value
            else:
                raise ValueError('The value of the description parameter is too long. Max length is {}'.format(
                    DESCRIPTION_MAX_LENGTH))
        else:
            raise ValueError('Invalid description value')

    @property
    def deal(self):
        return self.__deal

    @deal.setter
    def deal(self, value):
        if isinstance(value, dict):
            self.__deal = PayoutDealInfo(value)
        elif isinstance(value, PayoutDealInfo):
            self.__deal = value
        else:
            raise TypeError('Invalid deal type')

    @property
    def metadata(self):
        return self.__metadata

    @metadata.setter
    def metadata(self, value):
        if type(value) is dict:
            self.__metadata = value

    def validate(self):
        amount = self.amount
        if amount is None:
            self.__set_validation_error('Payout amount not specified')

        if amount.value <= 0.0:
            self.__set_validation_error('Invalid payout amount value: ' + str(amount.value))

        if self.payout_token:
            if self.payout_destination_data:
                self.__set_validation_error('Both payout_token and payout_destination_data values are specified')

        elif self.payout_token is None:
            if self.payout_destination_data is None:
                self.__set_validation_error('Both payout_token and payout_destination_data values are not specified')

    def __set_validation_error(self, message):
        raise ValueError(message)



# -*- coding: utf-8 -*-
import re

from yookassa.domain.common import BaseObject


class CreditCard(BaseObject):
    # Первые 6 цифр номера карты (BIN).
    __first6 = None
    # Последние 4 цифры номера карты.
    __last4 = None
    # Срок действия, год, YYYY.
    __expiry_year = None
    # Срок действия, месяц, MM.
    __expiry_month = None
    # Тип банковской карты. Возможные значения:
    # ~`MasterCard` (для карт Mastercard и Maestro), ~`Visa` (для карт Visa и Visa Electron),
    # ~`Mir`, ~`UnionPay`, ~`JCB`, ~`AmericanExpress`, ~`DinersClub` и ~`Unknown`.
    __card_type = None
    # Код страны, в которой выпущена карта.
    # Передается в формате [ISO-3166 alpha-2](https://www.iso.org/obp/ui/#iso:pub:PUB500001:en).
    # Пример: RU.
    __issuer_country = None
    # Наименование банка, выпустившего карту.
    __issuer_name = None
    # Источник данных банковской карты.
    # Возможные значения: ~`apple_pay`, ~`google_pay`, ~`mir_pay`.
    # Присутствует, если пользователь при оплате выбрал карту, сохраненную в Apple Pay, Google Pay или MirPay.
    __source = None

    @property
    def first6(self):
        return self.__first6

    @first6.setter
    def first6(self, value):
        cast_value = str(value)
        if re.match('^[0-9]{6}$', cast_value):
            self.__first6 = cast_value
        else:
            raise ValueError('Invalid first6 value')

    @property
    def last4(self):
        return self.__last4

    @last4.setter
    def last4(self, value):
        cast_value = str(value)
        if re.match(r'^[\d]{4}$', cast_value):
            self.__last4 = cast_value
        else:
            raise ValueError('Invalid last4 value')

    @property
    def expiry_year(self):
        return self.__expiry_year

    @expiry_year.setter
    def expiry_year(self, value):
        cast_value = str(value)
        if re.match(r'^\d\d\d\d$', cast_value) and 2000 < int(cast_value) < 2200:
            self.__expiry_year = cast_value
        else:
            raise ValueError('Invalid card expiry year value')

    @property
    def expiry_month(self):
        return self.__expiry_month

    @expiry_month.setter
    def expiry_month(self, value):
        cast_value = str(value)
        if re.match(r'^\d\d$', cast_value) and 0 < int(cast_value) <= 12:
            self.__expiry_month = cast_value
        else:
            raise ValueError('Invalid card expiry month value')

    @property
    def card_type(self):
        return self.__card_type

    @card_type.setter
    def card_type(self, value):
        self.__card_type = value

    @property
    def issuer_country(self):
        return self.__issuer_country

    @issuer_country.setter
    def issuer_country(self, value):
        cast_value = str(value)
        if len(cast_value) == 2:
            self.__issuer_country = cast_value
        else:
            raise ValueError('Invalid card issuer country value')

    @property
    def issuer_name(self):
        return self.__issuer_name

    @issuer_name.setter
    def issuer_name(self, value):
        self.__issuer_name = str(value)

    @property
    def source(self):
        return self.__source

    @source.setter
    def source(self, value):
        self.__source = str(value)



# -*- coding: utf-8 -*-
"""Top-level package for YooKassa API Python Client Library."""

from yookassa.configuration import Configuration
from yookassa.deal import Deal
from yookassa.payment import Payment
from yookassa.payout import Payout
from yookassa.receipt import Receipt
from yookassa.refund import Refund
from yookassa.settings import Settings
from yookassa.webhook import Webhook

__author__ = "YooMoney"
__email__ = 'cms@yoomoney.ru'
__version__ = '2.3.3'



# -*- coding: utf-8 -*-
from yookassa.domain.common.confirmation_type import ConfirmationType
from yookassa.domain.models.confirmation.request.confirmation_request import ConfirmationRequest


class ConfirmationRedirect(ConfirmationRequest):
    """
    Class representing redirect confirmation data object
    """
    __return_url = None

    __enforce = None

    def __init__(self, *args, **kwargs):
        super(ConfirmationRedirect, self).__init__(*args, **kwargs)
        if self.type is None or self.type is not ConfirmationType.REDIRECT:
            self.type = ConfirmationType.REDIRECT

    @property
    def return_url(self):
        return self.__return_url

    @return_url.setter
    def return_url(self, value):
        cast_value = str(value)
        if cast_value:
            self.__return_url = cast_value
        else:
            raise ValueError('Invalid returnUrl value')

    @property
    def enforce(self):
        return self.__enforce

    @enforce.setter
    def enforce(self, value):
        self.__enforce = bool(value)



# -*- coding: utf-8 -*-
from deprecated import deprecated

from yookassa.domain.common.payment_method_type import PaymentMethodType
from yookassa.domain.models.payment_data.payment_data import ResponsePaymentData


@deprecated("This class will be removed in one of future versions")
class PaymentDataWechat(ResponsePaymentData):
    def __init__(self, *args, **kwargs):
        super(PaymentDataWechat, self).__init__(*args, **kwargs)
        if self.type is None or self.type is not PaymentMethodType.WECHAT:
            self.type = PaymentMethodType.WECHAT



# -*- coding: utf-8 -*-
from yookassa.domain.common import BaseObject


class AuthorizationDetails(BaseObject):
    """
       Class representing authorization details data wrapper object
       """
    __rrn = None

    __auth_code = None

    __three_d_secure = None

    @property
    def rrn(self):
        return self.__rrn

    @rrn.setter
    def rrn(self, value):
        self.__rrn = str(value)

    @property
    def auth_code(self):
        return self.__auth_code

    @auth_code.setter
    def auth_code(self, value):
        self.__auth_code = str(value)

    @property
    def three_d_secure(self):
        return self.__three_d_secure

    @three_d_secure.setter
    def three_d_secure(self, value):
        if isinstance(value, dict):
            self.__three_d_secure = ThreeDSecure(value)
        elif isinstance(value, ThreeDSecure):
            self.__three_d_secure = value
        else:
            raise TypeError('Invalid three_d_secure value type')


class ThreeDSecure(BaseObject):
    """
    Class representing 3‑D Secure user authentication details data wrapper object
    """
    __applied = None

    @property
    def applied(self):
        return self.__applied

    @applied.setter
    def applied(self, value):
        self.__applied = bool(value)



# -*- coding: utf-8 -*-
from yookassa.domain.common import BaseObject
from yookassa.domain.notification.webhook_notification_types import WebhookNotificationType, \
    WebhookNotificationEventType
from yookassa.domain.response import DealResponse
from yookassa.domain.response.payment_response import PaymentResponse
from yookassa.domain.response.payout_response import PayoutResponse
from yookassa.domain.response.refund_response import RefundResponse


class WebhookNotification(BaseObject):

    __type = None

    __event = None

    __object = None

    @property
    def type(self):
        return self.__type

    @type.setter
    def type(self, value):
        self.__type = value

    @property
    def event(self):
        return self.__event

    @event.setter
    def event(self, value):
        self.__event = value

    @property
    def object(self):
        return self.__object

    @object.setter
    def object(self, value):
        if isinstance(value, dict) and value:
            self.__object = PaymentResponse(value)
        elif not value:
            raise ValueError('Parameter object is empty')
        else:
            raise TypeError('Invalid object type')


class PaymentWebhookNotification(WebhookNotification):
    pass


class RefundWebhookNotification(WebhookNotification):

    @property
    def object(self):
        return self.__object

    @object.setter
    def object(self, value):
        if isinstance(value, dict) and value:
            self.__object = RefundResponse(value)
        elif not value:
            raise ValueError('Parameter object is empty')
        else:
            raise TypeError('Invalid object type')


class DealWebhookNotification(WebhookNotification):

    @property
    def object(self):
        return self.__object

    @object.setter
    def object(self, value):
        if isinstance(value, dict) and value:
            self.__object = DealResponse(value)
        elif not value:
            raise ValueError('Parameter object is empty')
        else:
            raise TypeError('Invalid object type')


class PayoutWebhookNotification(WebhookNotification):

    @property
    def object(self):
        return self.__object

    @object.setter
    def object(self, value):
        if isinstance(value, dict) and value:
            self.__object = PayoutResponse(value)
        elif not value:
            raise ValueError('Parameter object is empty')
        else:
            raise TypeError('Invalid object type')


class WebhookNotificationFactory(object):
    """
    Base factory class for object that has type property.
    """

    def create(self, data):
        """
        Create instance from value

        :param data: dictionary that has type key
        :return: Typed object instance
        """
        if isinstance(data, dict):
            if 'type' in data and 'event' in data:
                return self.__get_instance(data)
            else:
                raise ValueError('Parameter "data" should contain "type" and "event" fields')
        else:
            raise TypeError('Parameter "data" should be "dict"')

    def __get_instance(self, data):
        class_object = self.__get_class_object(data)
        return class_object(data)

    @staticmethod
    def __get_class_object(data):
        if data['type'] == WebhookNotificationType.NOTIFICATION:
            if data['event'] == WebhookNotificationEventType.REFUND_SUCCEEDED:
                return RefundWebhookNotification
            elif data['event'] == WebhookNotificationEventType.DEAL_CLOSED:
                return DealWebhookNotification
            elif data['event'] == WebhookNotificationEventType.PAYOUT_SUCCEEDED or \
                    data['event'] == WebhookNotificationEventType.PAYOUT_CANCELED:
                return PayoutWebhookNotification
            else:
                return WebhookNotification
        else:
            raise TypeError('Parameter "data" should contain "type" field')



# -*- coding: utf-8 -*-
from yookassa.domain.common import BaseObject


class Recipient(BaseObject):
    """
    Class representing recipient data wrapper object
    """
    __account_id = None

    __gateway_id = None

    @property
    def account_id(self):
        return self.__account_id

    @account_id.setter
    def account_id(self, value):
        self.__account_id = str(value)

    @property
    def gateway_id(self):
        return self.__gateway_id

    @gateway_id.setter
    def gateway_id(self, value):
        self.__gateway_id = str(value)



# -*- coding: utf-8 -*-
from yookassa.domain.common.response_object import ResponseObject
from yookassa.domain.models.amount import Amount
from yookassa.domain.models.deal import RefundDealInfo
from yookassa.domain.models.refund_source import RefundSource


class RefundResponse(ResponseObject):
    """
    Class representing response object.

    Contains data
    """
    __id = None

    __payment_id = None

    __status = None

    __receipt_registration = None

    __created_at = None

    __amount = None

    __description = None

    __sources = None

    __deal = None

    @property
    def id(self):
        return self.__id

    @id.setter
    def id(self, value):
        self.__id = value

    @property
    def payment_id(self):
        return self.__payment_id

    @payment_id.setter
    def payment_id(self, value):
        self.__payment_id = value

    @property
    def status(self):
        return self.__status

    @status.setter
    def status(self, value):
        self.__status = value

    @property
    def receipt_registration(self):
        return self.__receipt_registration

    @receipt_registration.setter
    def receipt_registration(self, value):
        self.__receipt_registration = value

    @property
    def created_at(self):
        return self.__created_at

    @created_at.setter
    def created_at(self, value):
        self.__created_at = value

    @property
    def amount(self):
        return self.__amount

    @amount.setter
    def amount(self, value):
        self.__amount = Amount(value)

    @property
    def description(self):
        return self.__description

    @description.setter
    def description(self, value):
        self.__description = value

    @property
    def sources(self):
        return self.__sources

    @sources.setter
    def sources(self, value):
        if isinstance(value, list):
            self.__sources = [RefundSource(item) for item in value]
        else:
            self.__sources = value

    @property
    def deal(self):
        return self.__deal

    @deal.setter
    def deal(self, value):
        self.__deal = RefundDealInfo(value)



# -*- coding: utf-8 -*-
from yookassa.domain.request.receipt_request import ReceiptRequest


class ReceiptRequestBuilder(object):
    def __init__(self):
        self.__request = ReceiptRequest()

    def set_type(self, value):
        self.__request.type = value
        return self

    def set_send(self, value):
        self.__request.send = value
        return self

    def set_customer(self, value):
        self.__request.customer = value
        return self

    def set_tax_system_code(self, value):
        self.__request.tax_system_code = value
        return self

    def set_items(self, value):
        self.__request.items = value
        return self

    def set_settlements(self, value):
        self.__request.settlements = value
        return self

    def set_payment_id(self, value):
        self.__request.payment_id = value
        return self

    def set_refund_id(self, value):
        self.__request.refund_id = value
        return self

    def set_on_behalf_of(self, value):
        self.__request.on_behalf_of = value
        return self

    def build(self):
        return self.__request



# -*- coding: utf-8 -*-
from yookassa.domain.common.response_object import ResponseObject


class WebhookResponse(ResponseObject):
    __id = None

    __url = None

    __event = None

    @property
    def id(self):
        return self.__id

    @id.setter
    def id(self, value):
        self.__id = value

    @property
    def event(self):
        return self.__event

    @event.setter
    def event(self, value):
        self.__event = value

    @property
    def url(self):
        return self.__url

    @url.setter
    def url(self, value):
        self.__url = value


class WebhookList(ResponseObject):
    __type = None

    __items = None

    @property
    def type(self):
        return self.__type

    @type.setter
    def type(self, value):
        self.__type = value

    @property
    def items(self):
        return self.__items

    @items.setter
    def items(self, value):
        if isinstance(value, list):
            self.__items = [WebhookResponse(webhook) for webhook in value]
        else:
            self.__items = value






# -*- coding: utf-8 -*-
from yookassa.domain.common import BaseObject


class PaymentData(BaseObject):
    """
    Base class for Payment data objects
    """
    __type = None

    @property
    def type(self):
        return self.__type

    @type.setter
    def type(self, value):
        self.__type = str(value)


class ResponsePaymentData(PaymentData):
    __id = None

    __saved = None

    __title = None

    @property
    def id(self):
        return self.__id

    @id.setter
    def id(self, value):
        self.__id = str(value)

    @property
    def saved(self):
        return self.__saved

    @saved.setter
    def saved(self, value):
        self.__saved = bool(value)

    @property
    def title(self):
        return self.__title

    @title.setter
    def title(self, value):
        self.__title = str(value)




# -*- coding: utf-8 -*-
from yookassa.domain.common import BaseObject
from yookassa.domain.models import Settlement


class PaymentDealInfo(BaseObject):
    """
    Class representing PaymentDeal data wrapper object
    """
    __id = None

    __settlements = []

    @property
    def id(self):
        return self.__id

    @id.setter
    def id(self, value):
        self.__id = value

    @property
    def settlements(self):
        return self.__settlements

    @settlements.setter
    def settlements(self, value):
        if isinstance(value, list):
            settlements = []
            for item in value:
                if isinstance(item, dict):
                    settlements.append(Settlement(item))
                elif isinstance(item, Settlement):
                    settlements.append(item)
                else:
                    raise TypeError('Invalid item type in deal.settlements')

            self.__settlements = settlements
        elif value is None:
            self.__settlements = []
        else:
            raise TypeError('Invalid settlements value type in PaymentDealInfo')


class PayoutDealInfo(BaseObject):
    """
    Class representing PayoutDeal data wrapper object
    """
    __id = None

    @property
    def id(self):
        return self.__id

    @id.setter
    def id(self, value):
        self.__id = value


class RefundDealInfo(BaseObject):
    """
    Class representing PaymentDeal data wrapper object
    """
    __id = None

    __refund_settlements = []

    @property
    def id(self):
        return self.__id

    @id.setter
    def id(self, value):
        self.__id = value

    @property
    def refund_settlements(self):
        return self.__refund_settlements

    @refund_settlements.setter
    def refund_settlements(self, value):
        if isinstance(value, list):
            settlements = []
            for item in value:
                if isinstance(item, dict):
                    settlements.append(Settlement(item))
                elif isinstance(item, Settlement):
                    settlements.append(item)
                else:
                    raise TypeError('Invalid item type in deal.refund_settlements')

            self.__refund_settlements = settlements
        elif value is None:
            self.__refund_settlements = []
        else:
            raise TypeError('Invalid refund_settlements value type in RefundDealInfo')


class RefundDealData(BaseObject):
    """
    Class representing PaymentDeal data wrapper object
    """
    __refund_settlements = []

    @property
    def refund_settlements(self):
        return self.__refund_settlements

    @refund_settlements.setter
    def refund_settlements(self, value):
        if isinstance(value, list):
            settlements = []
            for item in value:
                if isinstance(item, dict):
                    settlements.append(Settlement(item))
                elif isinstance(item, Settlement):
                    settlements.append(item)
                else:
                    raise TypeError('Invalid item type in deal.refund_settlements')

            self.__refund_settlements = settlements
        elif value is None:
            self.__refund_settlements = []
        else:
            raise TypeError('Invalid refund_settlements value type in RefundDealData')


class DealType:
    """
    Constants representing deal types. Available values are:
    """
    SAFE_DEAL = 'safe_deal'


class DealStatus:
    """
    Constants representing deal statuses. Available values are:
    """
    OPENED = 'opened'
    CLOSED = 'closed'


class FeeMoment:
    """
    Constants representing fee moment types. Available values are:
    """
    PAYMENT_SUCCEEDED = 'payment_succeeded'
    DEAL_CLOSED = 'deal_closed'



# -*- coding: utf-8 -*-
import re

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages

long_description = """
# YooKassa API Python Client Library

[![Build Status](https://travis-ci.org/yoomoney/yookassa-sdk-python.svg?branch=master)](https://travis-ci.org/yoomoney/yookassa-sdk-python)
[![Latest Stable Version](https://img.shields.io/pypi/v/yookassa.svg)](https://pypi.org/project/yookassa/)
[![Total Downloads](https://img.shields.io/pypi/dm/yookassa.svg)](https://pypi.org/project/yookassa/)
[![License](https://img.shields.io/pypi/l/yookassa.svg)](https://git.yoomoney.ru/projects/SDK/repos/yookassa-sdk-python)

[Russian](README.md) | English

This product is used for managing payments under [The YooKassa API](https://yookassa.ru/en/developers/api)
For usage by those who implemented YooKassa using the API method.

## Requirements
1. Python 2.7 or Python 3.x
2. pip

## Installation
### Under console using pip

1. Install pip.
2. In the console, run the following command:
```bash
pip install --upgrade yookassa
```

### Under console using easy_install
1. Install easy_install.
2. In the console, run the following command:
```bash
easy_install --upgrade yookassa
```

### Manually

1. In the console, run the following command:
```bash
wget https://pypi.python.org/packages/5a/be/5eafdfb14aa6f32107e9feb6514ca1ad3fe56f8e5ee59d20693b32f7e79f/yookassa-1.0.0.tar.gz#md5=46595279b5578fd82a199bfd4cd51db2
tar zxf yookassa-1.0.0.tar.gz
cd yookassa-1.0.0
python setup.py install
```


## Commencing work

1. Import module
```python
import yookassa
```

2. Configure a Client
```python
from yookassa import Configuration

Configuration.configure('<Account Id>', '<Secret Key>')
```

or

```python
from yookassa import Configuration

Configuration.account_id = '<Account Id>'
Configuration.secret_key = '<Secret Key>'
```

or via oauth

```python
from yookassa import Configuration

Configuration.configure_auth_token('<Oauth Token>')
```

If you agree to participate in the development of the SDK, you can submit data about your framework, cms or module:

```python
from yookassa import Configuration
from yookassa.domain.common.user_agent import Version

Configuration.configure('<Account Id>', '<Secret Key>')
Configuration.configure_user_agent(
    framework=Version('Django', '2.2.3'),
    cms=Version('Wagtail', '2.6.2'),
    module=Version('Y.CMS', '0.0.1')
)
```

3. Call the required API method. [More details in our documentation for the YooKassa API](https://yookassa.ru/en/developers/api)
"""

with open('src/yookassa/__init__.py') as fp:
    version = re.search(r"__version__\s*=\s*'(.*)'", fp.read()).group(1)

setup(
    name="yookassa",
    author="YooMoney",
    author_email="cms@yoomoney.ru",
    version=version,
    keywords="yoomoney, yookassa, payout, sdk, python",
    description="YooKassa API SDK Python Library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://git.yoomoney.ru/projects/SDK/repos/yookassa-sdk-python",
    package_dir={"": "src"},
    packages=find_packages('src'),
    install_requires=["requests", "urllib3", "netaddr", "distro", "deprecated"],
    zip_safe=False,
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9"
    ]
)



# -*- coding: utf-8 -*-
from yookassa.domain.common import ResponseObject
from yookassa.domain.models import Settlement
from yookassa.domain.response import ReceiptItemResponse


class ReceiptResponse(ResponseObject):
    """
    Class representing response object.

    Contains data
    """
    __id = None

    __type = None

    __refund_id = None

    __payment_id = None

    __status = None

    __fiscal_document_number = None

    __fiscal_storage_number = None

    __fiscal_attribute = None

    __registered_at = None

    __fiscal_provider_id = None

    __tax_system_code = None

    __items = []

    __settlements = []

    __on_behalf_of = None

    @property
    def id(self):
        return self.__id

    @id.setter
    def id(self, value):
        self.__id = str(value)

    @property
    def type(self):
        return self.__type

    @type.setter
    def type(self, value):
        self.__type = str(value)

    @property
    def payment_id(self):
        return self.__payment_id

    @payment_id.setter
    def payment_id(self, value):
        self.__payment_id = str(value)

    @property
    def refund_id(self):
        return self.__refund_id

    @refund_id.setter
    def refund_id(self, value):
        self.__refund_id = str(value)

    @property
    def status(self):
        return self.__status

    @status.setter
    def status(self, value):
        self.__status = str(value)

    @property
    def receipt_registration(self):
        return self.status

    @receipt_registration.setter
    def receipt_registration(self, value):
        self.status = value

    @property
    def fiscal_document_number(self):
        return self.__fiscal_document_number

    @fiscal_document_number.setter
    def fiscal_document_number(self, value):
        self.__fiscal_document_number = str(value)

    @property
    def fiscal_storage_number(self):
        return self.__fiscal_storage_number

    @fiscal_storage_number.setter
    def fiscal_storage_number(self, value):
        self.__fiscal_storage_number = str(value)

    @property
    def fiscal_attribute(self):
        return self.__fiscal_attribute

    @fiscal_attribute.setter
    def fiscal_attribute(self, value):
        self.__fiscal_attribute = str(value)

    @property
    def registered_at(self):
        return self.__registered_at

    @registered_at.setter
    def registered_at(self, value):
        self.__registered_at = value

    @property
    def fiscal_provider_id(self):
        return self.__fiscal_provider_id

    @fiscal_provider_id.setter
    def fiscal_provider_id(self, value):
        self.__fiscal_provider_id = str(value)

    @property
    def tax_system_code(self):
        return self.__tax_system_code

    @tax_system_code.setter
    def tax_system_code(self, value):
        self.__tax_system_code = int(value)

    @property
    def items(self):
        return self.__items

    @items.setter
    def items(self, value):
        if isinstance(value, list):
            self.__items = [ReceiptItemResponse(item) for item in value]
        else:
            self.__items = []

    @property
    def settlements(self):
        return self.__settlements

    @settlements.setter
    def settlements(self, value):
        if isinstance(value, list):
            self.__settlements = [Settlement(item) for item in value]
        else:
            self.__settlements = []

    @property
    def on_behalf_of(self):
        return self.__on_behalf_of

    @on_behalf_of.setter
    def on_behalf_of(self, value):
        self.__on_behalf_of = str(value)



from yookassa.domain.models.airline import Airline, Passenger, Leg
from yookassa.domain.models.amount import Amount
from yookassa.domain.models.authorization_details import AuthorizationDetails
from yookassa.domain.models.cancellation_details import CancellationDetails, CancellationDetailsPartyCode, \
    CancellationDetailsReasonCode, PayoutCancellationDetailsReasonCode
from yookassa.domain.models.currency import Currency
from yookassa.domain.models.receipt import Receipt, ReceiptCustomer, ReceiptItem
from yookassa.domain.models.receipt_item_supplier import ReceiptItemSupplier
from yookassa.domain.models.recipient import Recipient
from yookassa.domain.models.refund_source import RefundSource
from yookassa.domain.models.requestor import Requestor, RequestorType, RequestorMerchant, RequestorFactory, \
    RequestorThirdPartyClient
from yookassa.domain.models.settlement import Settlement, SettlementType, SettlementPayoutType
from yookassa.domain.models.transfer import Transfer
from yookassa.domain.models.deal import DealStatus, DealType, FeeMoment, PaymentDealInfo, PayoutDealInfo, \
    RefundDealInfo, RefundDealData



# -*- coding: utf-8 -*-
from yookassa.domain.common.type_factory import TypeFactory
from yookassa.domain.models.payment_data.payment_data_class_map import PaymentDataClassMap


class PaymentDataFactory(TypeFactory):
    """
    Factory for payment data objects
    """

    def __init__(self):
        super(PaymentDataFactory, self).__init__(PaymentDataClassMap())



# -*- coding: utf-8 -*-


class HttpVerb(object):
    """
    Constants representing http method verbs. Available values are:

    * yookassa.domain.common.HttpVerb.GET
    * yookassa.domain.common.HttpVerb.POST
    * yookassa.domain.common.HttpVerb.PUT
    * yookassa.domain.common.HttpVerb.PATCH
    * yookassa.domain.common.HttpVerb.HEAD
    * yookassa.domain.common.HttpVerb.OPTIONS
    * yookassa.domain.common.HttpVerb.DELETE
    """
    GET = 'get'
    POST = 'post'
    PUT = 'put'
    PATCH = 'patch'
    HEAD = 'head'
    OPTIONS = 'options'
    DELETE = 'delete'
